%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% writePosFileFromVMR.m
% Purpose: write BrainVoyager positioning file (*.pos) from anatomical (*.vmr) file (VMR = Volume based anatomical MRI data)
% Activate: load this script in Matlab and press 'run'
% HB 200619
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%clear; % clear workspace
h = msgbox('This script will write a BrainVoyager positioning file (*.pos) from an anatomical (*.vmr) file', 'Write BrainVoyager POS file');
uiwait(h);
%dataType = questdlg('Which datatype of *.vmr do you prefer to read (*.vmr (8 bit) or *.v16 (16 bit))? ','Choose *.vmr type','*.vmr (8 bit)','*.v16 (16 bit)','*.vmr (8 bit)');
%if strcmp(dataType,'*.vmr (8 bit)')
    vmr8bit = 1;
%else
%    vmr8bit = 0;
%end
if vmr8bit
    [FileName,PathName] = uigetfile('*.vmr','Please select the BrainVoyager *.vmr file...');
else
    [FileName,PathName] = uigetfile('*.v16','Please select the BrainVoyager *.v16 file...');
end
vmrFile = strcat(PathName, FileName);

fid=fopen(vmrFile,'rb');
VMRversion = 0;
BV_2000 = 1;
BV_QX = 2;
% VMR header
flag = fread(fid,[1 1], 'ushort');
if (flag > 10)
    VMRversion = BV_2000;
elseif (flag >= 2 && flag < 10)
    VMRversion = BV_QX;
else
    disp('VMR version unknown')
end

if (VMRversion == BV_2000)
    DimX = flag; % file version 1, no file format version indicated
elseif (VMRversion == BV_QX)
    DimX = fread(fid,[1 1],'ushort'); % file version 2 ( = BrainVoyager QX 1.0)
else
    NrOfLags = fread(fid,[1 1],'ushort'); % file version 0 (prehistoric BV)
    DimX = fread(fid,[1 1],'ushort');
end

DimY = fread(fid,[1 1],'ushort');
DimZ = fread(fid,[1 1],'ushort');
% VMR data
if vmr8bit
    vmrVolume1D = fread(fid,DimZ*DimY*DimX, 'uint8'); % normal *.vmr
else
    vmrVolume1D = fread(fid,DimZ*DimY*DimX,'uint16'); % *.v16
end

if (VMRversion>1)
    XOffset = fread(fid,[1 1], 'ushort');
    YOffset = fread(fid,[1 1], 'ushort');
    ZOffset = fread(fid,[1 1], 'ushort');
    framingCubeDim = fread(fid,[1 1], 'ushort');
    % scanner information, also saved in *.pos file format
    PosInfoVerified = fread(fid,[1 1], 'int32');
    CoordinateSystem = fread(fid,[1 1], 'int32');
    Slice1CenterX = fread(fid,[1 1], 'float32');
    Slice1CenterY = fread(fid,[1 1], 'float32');
    Slice1CenterZ = fread(fid,[1 1], 'float32');
    SliceNCenterX = fread(fid,[1 1], 'float32');
    SliceNCenterY = fread(fid,[1 1], 'float32');
    SliceNCenterZ = fread(fid,[1 1], 'float32');
    RowDirX = fread(fid,[1 1], 'float32');
    RowDirY = fread(fid,[1 1], 'float32');
    RowDirZ = fread(fid,[1 1], 'float32');
    ColDirX = fread(fid,[1 1], 'float32');
    ColDirY = fread(fid,[1 1], 'float32');
    ColDirZ = fread(fid,[1 1], 'float32');
    NrOfRows = fread(fid,[1 1], 'int32');
    NrOfCols = fread(fid,[1 1], 'int32');
    FoVRows = fread(fid,[1 1], 'float32');
    FoVCols = fread(fid,[1 1], 'float32');
    SliceThickness = fread(fid,[1 1], 'float32');
    GapThickness = fread(fid,[1 1], 'float32');
end
fclose(fid);

[pathstr,name,ext] = fileparts(vmrFile);
posFile = fullfile(PathName,[name '_VMR.pos']);
fid = fopen(posFile,'w');
fprintf(fid, '%s %16s\n', 'FileVersion:', num2str(3));
fprintf(fid, '%s %16s\n', 'ProjectType:', 'VMR');
fprintf(fid, '%s %17s\n', 'NrOfSlices:', num2str(DimZ));
fprintf(fid, '%s %14.6f\n', 'Slice1CenterX:', Slice1CenterX);
fprintf(fid, '%s %14.6f\n', 'Slice1CenterY:', Slice1CenterY);
fprintf(fid, '%s %14.6f\n', 'Slice1CenterZ:', Slice1CenterZ);
fprintf(fid, '%s %14.6f\n', 'SliceNCenterX:', SliceNCenterX);
fprintf(fid, '%s %14.6f\n', 'SliceNCenterY:', SliceNCenterY);
fprintf(fid, '%s %14.6f\n', 'SliceNCenterZ:', SliceNCenterZ);
fprintf(fid, '%s %19.6f\n', 'RowDirX: ', RowDirX);
fprintf(fid, '%s %19.6f\n', 'RowDirY: ', RowDirY);
fprintf(fid, '%s %19.6f\n', 'RowDirZ: ', RowDirZ);
fprintf(fid, '%s %19.6f\n', 'ColDirX: ', ColDirX);
fprintf(fid, '%s %19.6f\n', 'ColDirY: ', ColDirY);
fprintf(fid, '%s %19.6f\n', 'ColDirZ: ', ColDirZ);
fprintf(fid, '%s %16s\n', 'MatrixRows: ', num2str(NrOfRows));
fprintf(fid, '%s %16s\n', 'MatrixCols: ', num2str(NrOfCols));
fprintf(fid, '%s %19s\n', 'FoVRows: ', num2str(FoVRows));
fprintf(fid, '%s %19s\n', 'FoVCols: ', num2str(FoVCols));
fprintf(fid, '%s %13.6f\n', 'SliceThickness:', SliceThickness);
fprintf(fid, '%s %15.6f\n', 'GapThickness:', GapThickness);
fprintf(fid, '%s %11s\n\n', 'CoordinateSystem:', num2str(1)); 
fclose(fid);

msgbox(strcat('Positioning file finished'));

